﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.SearchModels;
using Microsoft.Extensions.Logging;
using System.Diagnostics;

namespace FlowerShop
{
    public partial class FormCreateOrder : Form
    {
        private readonly ILogger _logger;
        private readonly IBouquetLogic _logicBouquet;
        private readonly IOrderLogic _logicOrder;

        public FormCreateOrder(ILogger<FormCreateOrder> logger, IBouquetLogic logicBouquet, IOrderLogic logicOrder)
        {
            InitializeComponent();
            _logger = logger;
            _logicBouquet = logicBouquet;
            _logicOrder = logicOrder;
            LoadData();
        }

        private void LoadData()
        {
            _logger.LogInformation("Receiving bouquets for order");

            try
            {
                var list = _logicBouquet.ReadList(null);
                if (list != null)
                {
                    comboBoxBouquet.DisplayMember = "BouquetName";
                    comboBoxBouquet.ValueMember = "Id";
                    comboBoxBouquet.DataSource = list;
                    comboBoxBouquet.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during receiving bouquets for order");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void OrderForm_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void CalculateSum()
        {
            if (comboBoxBouquet.SelectedValue != null && !string.IsNullOrEmpty(textBoxCount.Text))
            {
                try
                {
                    int id = Convert.ToInt32(comboBoxBouquet.SelectedValue);
                    var furniture = _logicBouquet.ReadElement(new BouquetSearchModel { Id = id });
                    int count = Convert.ToInt32(textBoxCount.Text);

                    textBoxSum.Text = Math.Round(count * (furniture?.Price ?? 0), 2).ToString();

                    _logger.LogInformation("Calculating order's sum");
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Error during calculating order's sum");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void textBoxCount_TextChanged(object sender, EventArgs e)
        {
            CalculateSum();
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (comboBoxBouquet.SelectedValue == null)
            {
                MessageBox.Show("Выберите изделие", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Создание заказа");
            try
            {
                var operationResult = _logicOrder.CreateOrder(new OrderBindingModel
                {
                    BouquetId = Convert.ToInt32(comboBoxBouquet.SelectedValue),
                    Count = Convert.ToInt32(textBoxCount.Text),
                    Sum = Convert.ToDouble(textBoxSum.Text)
                });

                if (!operationResult)
                {
                    throw new Exception("Error during order creation");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during order creation");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
