﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.SearchModels;
using FlowerShopDataModels.Models;
using Microsoft.Extensions.Logging;

namespace FlowerShop
{
    public partial class FormShop : Form
    {
        
        private readonly IShopLogic _logic;
        private readonly ILogger _logger;
        private Dictionary<int, (IBouquetModel, int)> _shopBouquets;
        private int? _id;
        public int Id { set { _id = value; } }

        public FormShop(ILogger<FormShop> logger, IShopLogic logic)
        {
            InitializeComponent(); 
            _logger = logger;
            _logic = logic;
            _shopBouquets = new();
        }

        private void FormShop_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                _logger.LogInformation("Loading shop");

                try
                {
                    var view = _logic.ReadElement(new ShopSearchModel { Id = _id.Value });

                    if (view != null)
                    {
                        textBoxName.Text = view.Name;
                        textBoxAddress.Text = view.Address;
                        openDateTimePicker.Value = view.OpeningDate;
                        textBoxMaxCount.Text = view.MaxCountBouquets.ToString();
                        _shopBouquets = view.ShopBouquets ?? new Dictionary<int, (IBouquetModel, int)>();
                        LoadData();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Error during loading shop");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void LoadData()
        {
            _logger.LogInformation("Loading shop's bouquets");

            try
            {
                if (_shopBouquets != null)
                {
                    dataGridView.Rows.Clear();
                    foreach (var bouquet in _shopBouquets)
                    {
                        dataGridView.Rows.Add(new object[] { bouquet.Key, bouquet.Value.Item1.BouquetName, bouquet.Value.Item2 });
                    }
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during loading shop's bouquets");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxName.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (string.IsNullOrEmpty(textBoxAddress.Text))
            {
                MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (string.IsNullOrEmpty(textBoxMaxCount.Text))
            {
                MessageBox.Show("Заполните макс. количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Saving shop");

            try
            {
                var model = new ShopBindingModel
                {
                    Id = _id ?? 0,
                    Name = textBoxName.Text,
                    Address = textBoxAddress.Text,
                    OpeningDate = openDateTimePicker.Value,
                    MaxCountBouquets = Convert.ToInt32(textBoxMaxCount.Text),
                    ShopBouquets = _shopBouquets
                };

                var operationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);

                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error during saving shop");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
