﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDataModels.Models;
using FlowerShopDataModels.Enums;
using Microsoft.Extensions.Logging;
using System.Text;

namespace FlowerShopBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly IShopStorage _shopStorage;
        private readonly IShopLogic _shopLogic;
        private readonly IBouquetStorage _bouquetStorage;

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IShopLogic shopLogic, IBouquetStorage bouquetStorage, IShopStorage shopStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _shopLogic = shopLogic;
            _bouquetStorage = bouquetStorage;
            _shopStorage = shopStorage;
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id: {Id}", model?.Id);

            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);

            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }

            model.Status = OrderStatus.Принят;

            if (_orderStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            return true;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            CheckModel(model, false);

            if (_orderStorage.GetElement(new OrderSearchModel { Id=model.Id })?.Status != OrderStatus.Принят)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }

            model.Status = OrderStatus.Выполняется;

            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
            }

            return true;
        }
        public bool FinishOrder(OrderBindingModel model)
        {
            CheckModel(model, false);

            var order = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (order?.Status != OrderStatus.Выполняется)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }

            var bouquet = _bouquetStorage.GetElement(new BouquetSearchModel() { Id = order.BouquetId });
            if (bouquet == null)
            {
                _logger.LogWarning("Status update to Готов operation failed. Bouquet not found");
                return false;
            }

            if (!CheckThenSupplyMany(bouquet, order.Count))
            {
                _logger.LogWarning("Status update to Готов operation failed. Shop supply error.");
                throw new Exception("В магазинах не хватает места!");
            }

            model.Status = OrderStatus.Готов;
            model.DateImplement = DateTime.Now;

            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
            }

            return true;
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            CheckModel(model, false);

            var order = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });

            if (order?.Status != OrderStatus.Готов)
            {
                _logger.LogWarning("Invalid order status");
                return false;
            }

            model.Status = OrderStatus.Выдан;
            model.DateImplement = order.DateImplement;

            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
            }

            return true;
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (!withParams)
            {
                return;
            }

            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Стоимость должна быть больше 0", nameof(model.Sum));
            }

            _logger.LogInformation("Order. Id: {Id}. Sum: {Sum}. BouquetId: {BouquetId}", model.Id, model.Sum, model.BouquetId);

            var element = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (element != null && element.Id == model.Id)
            {
                throw new InvalidOperationException("Заказ с таким идентификатором уже есть");
            }
        }

        public bool CheckThenSupplyMany(IBouquetModel bouquet, int count)
        {
            if (count <= 0)
            {
                _logger.LogWarning("Check then supply operation error. Bouquet count < 0.");
                return false;
            }

            int freeSpace = 0;
            foreach (var shop in _shopStorage.GetFullList())
            {
                freeSpace += shop.MaxCountBouquets;
                foreach (var bouq in shop.ShopBouquets)
                {
                    freeSpace -= bouq.Value.Item2;
                }
            }

            if (freeSpace - count < 0)
            {
                _logger.LogWarning("Check then supply operation error. There's no place for new bouquets in shops.");
                return false;
            }

            foreach (var shop in _shopStorage.GetFullList())
            {
                freeSpace = shop.MaxCountBouquets;

                foreach (var bouq in shop.ShopBouquets)
                {
                    freeSpace -= bouq.Value.Item2;
                }

                if (freeSpace == 0)
                {
                    continue;
                }

                if (freeSpace - count >= 0)
                {
                    if (_shopLogic.SupplyBouquets(new() { Id = shop.Id }, bouquet, count)) count = 0;
                    else
                    {
                        _logger.LogWarning("Supply error");
                        return false;
                    }
                }

                if (freeSpace - count < 0)
                {
                    if (_shopLogic.SupplyBouquets(new() { Id = shop.Id }, bouquet, freeSpace)) count -= freeSpace;
                    else
                    {
                        _logger.LogWarning("Supply error");
                        return false;
                    }
                }

                if (count <= 0)
                {
                    return true;
                }
            }

            return false;
        }
    }
}
